//
//  GS2BoxView.swift
//  GeometrySikenjo
//
//  Created by narita on 2018/04/03.
//  Copyright © 2018年 narita. All rights reserved.
//

import Cocoa

class GS2BoxView: NSView
{
    // MARK: - class methods
    
    // MARK: - init methods
    
    // MARK: - dealloc
    
    // MARK: - NSCopying, hash, isEqual:
    
    // MARK: - NSCoding methods
    
    // MARK: - restorableState methods
    
    // MARK: - life cycle methods
    
    // MARK: - action methods

    @IBAction func reset(_ sender: Any?)
    {
        self.updateBoxSize()
    }

    // 垂直方向のレイアウト調整

    // 上辺に外接させる
    @IBAction func alignmentTop(_ sender: Any?)
    {
        let theLine = self.bounds.maxY - 50
        
        self.boxs = self.boxs.map()
        {
            return $0.alignmentTop(base: theLine)
        }
    }

    // 垂直方向にセンタリング
    @IBAction func alignmentVCenter(_ sender: Any?)
    {
        let theLine = self.bounds.midY
        
        self.boxs = self.boxs.map()
        {
            return $0.alignmentVCenter(base: theLine)
        }
    }

    // 底辺に内接させる
    @IBAction func alignmentBottom(_ sender: Any?)
    {
        let theLine = self.bounds.minY + 50
        
        self.boxs = self.boxs.map()
        {
            return $0.alignmentBottom(base: theLine)
        }
    }

    
    @IBAction func justifyTopToBottom(_ sender: Any?)
    {
        let theFirstBaseLine = self.bounds.maxY - 50// 50は補助線のライン
        let theDelta         = CGFloat(20.0 + 3.0)  // 矩形の大きさは、40x20。矩形と矩形の間は、3とする
        
        self.boxs = self.boxs.enumerated().map
        {
            return $0.element.alignmentTop(base: theFirstBaseLine - theDelta * CGFloat($0.offset))
        }
    }

    // 水平方向のレイアウト調整

    // 左辺に外接させる
    @IBAction func alignmentLeft(_ sender: Any?)
    {
        let theLine = self.bounds.minX + 50
        
        self.boxs = self.boxs.map()
        {
            return $0.alignmentLeft(base: theLine)
        }
    }
    
    // 水平方向にセンタリングさせる
    @IBAction func alignmentHCenter(_ sender: Any?)
    {
        let theLine = self.bounds.midX
        
        self.boxs = self.boxs.map()
        {
            return $0.alignmentHCenter(base: theLine)
        }
    }
    
    // 右辺に内接させる
    @IBAction func alignmentRight(_ sender: Any?)
    {
        let theLine = self.bounds.maxX - 50
        
        self.boxs = self.boxs.map()
        {
            return $0.alignmentRight(base: theLine)
        }
    }
    
    @IBAction func justifyLeftToRight(_ sender: Any?)
    {
        let theFirstBaseLine = self.bounds.minX + 50// 50は補助線のライン
        let theDelta         = CGFloat(40.0 + 3.0)  // 矩形の大きさは、40x20。矩形と矩形の間は、3とする
        
        self.boxs = self.boxs.enumerated().map
        {
            return $0.element.alignmentLeft(base: theFirstBaseLine + theDelta * CGFloat($0.offset))
        }
    }
    
    // MARK: - event handling methods
    
    override func layout()
    {
        super.layout()

        self.updateBoxSize()
    }
    
    // MARK: - drawing methods

    override func draw(_ dirtyRect: NSRect)
    {
        super.draw(dirtyRect)

        NSGraphicsContext.saveGraphicsState()

        // Drawing boxes
        NSColor.red.set()
        
        self.boxs.forEach()
        {
            NSBezierPath(rect: $0).stroke()
        }
        
        // 青で補助線を引く。viewの矩形の内側50point
        NSColor.blue.set()
        
        NSBezierPath( rect: self.bounds.insetBy(dx:50.0, dy:-1.0) ).stroke()
        NSBezierPath( rect: self.bounds.insetBy(dx:-1.0, dy:50.0) ).stroke()

        NSGraphicsContext.restoreGraphicsState()
    }

    // MARK: - delegate/datasource methods
    
    // MARK: - accessor methods (in pairs)

    var boxs: Array<CGRect> = [] {didSet{self.setNeedsDisplay(self.bounds)}}

    

    // MARK: - Utility methods

    // それぞれの箱の大きさは、Viewに依存する
    func updateBoxSize()
    {
        let theW = UInt32(self.bounds.width) - 40
        let theH = UInt32(self.bounds.height) - 20
    
        // 40x20の矩形を5個作る
        var theArray: Array<CGRect> = []
        
        theArray.append(CGRect(x: Int(arc4random_uniform(theW)), y: Int(arc4random_uniform(theH)), width: 40, height: 20))
        theArray.append(CGRect(x: Int(arc4random_uniform(theW)), y: Int(arc4random_uniform(theH)), width: 40, height: 20))
        theArray.append(CGRect(x: Int(arc4random_uniform(theW)), y: Int(arc4random_uniform(theH)), width: 40, height: 20))
        theArray.append(CGRect(x: Int(arc4random_uniform(theW)), y: Int(arc4random_uniform(theH)), width: 40, height: 20))
        theArray.append(CGRect(x: Int(arc4random_uniform(theW)), y: Int(arc4random_uniform(theH)), width: 40, height: 20))

        self.boxs = theArray
    }
    
}
