//
//  MINTLGeo.swift
//  GeometrySikenjo
//
//  Created by narita on 2018/04/03.
//  Copyright © 2018年 narita. All rights reserved.
//

import Cocoa
import CoreGraphics

extension CGRect
{
    // 中心点
    public var center: CGPoint
    {
        return CGPoint(x: self.midX, y: self.midY)
    }
    
    // 垂直方向 -------------------------------------
    // 上辺に外接させる
    public func outsideTop(base: CGRect) -> CGRect
    {
        let theStdSelf = self.standardized
        let theStdBase = base.standardized
        
        let theDeltaY = theStdBase.maxY - theStdSelf.minY
        
        return theStdSelf.offsetBy(dx:0.0,
                                   dy:theDeltaY)
    }
    
    // 上辺に内接させる
    public func insideTop(base: CGRect) -> CGRect
    {
        let theStdSelf = self.standardized
        let theStdBase = base.standardized
        
        let theDeltaY = theStdBase.maxY - theStdSelf.maxY
        
        return theStdSelf.offsetBy(dx:0.0,
                                   dy:theDeltaY)
    }

    // 垂直方向にセンタリング
    public func centringVertical(base: CGRect) -> CGRect
    {
        let theStdSelf = self.standardized
        let theStdBase = base.standardized

        let theDeltaY = theStdBase.midY - theStdSelf.midY
        
        return theStdSelf.offsetBy(dx:0.0,
                                   dy:theDeltaY)
    }

    // 底辺に内接させる
    public func insideBottom(base: CGRect) -> CGRect
    {
        let theStdSelf = self.standardized
        let theStdBase = base.standardized
        
        let theDeltaY = theStdBase.minY - theStdSelf.minY
        
        return theStdSelf.offsetBy(dx:0.0,
                                   dy:theDeltaY)
    }

    // 底辺に外接させる
    public func outsideBottom(base: CGRect) -> CGRect
    {
        let theStdSelf = self.standardized
        let theStdBase = base.standardized
        
        let theDeltaY = theStdBase.minY - theStdSelf.maxY
        
        return theStdSelf.offsetBy(dx:0.0,
                                   dy:theDeltaY)
    }

    
    // alignment
    
    public func alignmentTop(base: CGFloat) -> CGRect
    {
        let theStdSelf = self.standardized
        
        let theDeltaY = base - theStdSelf.maxY
        
        return theStdSelf.offsetBy(dx:0.0,
                                   dy:theDeltaY)
    }

    public func alignmentVCenter(base: CGFloat) -> CGRect
    {
        let theStdSelf = self.standardized
        
        let theDeltaY = base - theStdSelf.midY
        
        return theStdSelf.offsetBy(dx:0.0,
                                   dy:theDeltaY)
    }

    public func alignmentBottom(base: CGFloat) -> CGRect
    {
        let theStdSelf = self.standardized
        
        let theDeltaY = base - theStdSelf.minY
        
        return theStdSelf.offsetBy(dx:0.0,
                                   dy:theDeltaY)
    }

    // 水平方向方向 -------------------------------------
    
    // 左辺に外接させる
    public func outsideLeft(base: CGRect)  -> CGRect
    {
        let theStdSelf = self.standardized
        let theStdBase = base.standardized
        
        let theDeltaX = theStdBase.minX - theStdSelf.maxX

        return theStdSelf.offsetBy(dx:theDeltaX,
                                   dy:0.0)
    }
    
    // 左辺に内接させる
    public func insideLeft(base: CGRect)  -> CGRect
    {
        let theStdSelf = self.standardized
        let theStdBase = base.standardized
        
        let theDeltaX = theStdBase.minX - theStdSelf.minX

        return theStdSelf.offsetBy(dx:theDeltaX,
                                   dy:0.0)
    }
    
    // 水平方向にセンタリングさせる
    public func centringHorizontal (base: CGRect)  -> CGRect
    {
        let theStdSelf = self.standardized
        let theStdBase = base.standardized
        
        let theDeltaX = theStdBase.midX - theStdSelf.midX

        return theStdSelf.offsetBy(dx:theDeltaX,
                                   dy:0.0)
    }
    
    // 右辺に内接させる
    public func insideRight(base: CGRect)  -> CGRect
    {
        let theStdSelf = self.standardized
        let theStdBase = base.standardized
        
        let theDeltaX = theStdBase.maxX - theStdSelf.maxX

        return theStdSelf.offsetBy(dx:theDeltaX,
                                   dy:0.0)
    }
    
    // 右辺に外接させる
    public func outsideRight(base: CGRect) -> CGRect
    {
        let theStdSelf = self.standardized
        let theStdBase = base.standardized
        
        let theDeltaX = theStdBase.maxX - theStdSelf.minX

        return theStdSelf.offsetBy(dx:theDeltaX,
                                   dy:0.0)
    }
    
    // alignment
    
    public func alignmentLeft(base: CGFloat) -> CGRect
    {
        let theStdSelf = self.standardized
        
        let theDeltaX = base - theStdSelf.minX

        return theStdSelf.offsetBy(dx:theDeltaX,
                                   dy:0.0)
    }
    
    public func alignmentHCenter(base: CGFloat) -> CGRect
    {
        let theStdSelf = self.standardized
        
        let theDeltaX = base - theStdSelf.midX

        return theStdSelf.offsetBy(dx:theDeltaX,
                                   dy:0.0)
    }
    
    public func alignmentRight(base: CGFloat) -> CGRect
    {
        let theStdSelf = self.standardized
        
        let theDeltaX = base - theStdSelf.maxX

        return theStdSelf.offsetBy(dx:theDeltaX,
                                   dy:0.0)
    }

}
